@@ -317,7 +317,7 @@ class CrudManager(BaseManager):
 
     def _filter_kwargs(self, kwargs):
         """Drop null values and handle ids."""
-        for key, ref in six.iteritems(kwargs.copy()):
+        for key, ref in kwargs.copy().items():
             if ref is None:
                 kwargs.pop(key)
             else:
@@ -402,7 +402,7 @@ class CrudManager(BaseManager):
                 'name': self.resource_class.__name__,
                 'args': kwargs
             }
-            raise exceptions.NotFound(404, msg)
+            raise exceptions.NotFound(msg)
         elif num > 1:
             raise exceptions.NoUniqueMatch
         else:
@@ -476,7 +476,7 @@ class Resource(object):
         return None
 
     def _add_details(self, info):
-        for (k, v) in six.iteritems(info):
+        for (k, v) in info.items():
             try:
                 setattr(self, k, v)
                 self._info[k] = v
